import sys
import os
import uno
import uno.main as unomain
import unoforge

__version__ = '1.1.0'
__author__ = 'Gaming32'


def _exit_callback():
    for cb in unoforge._exit_callbacks:
        try:
            cb()
        except Exception:
            print('An error occurred in the mod "%s"' % cb.__module__)
            print('More info:\n   ', *sys.exc_info()[:2])


def getmods():
    if os.path.isdir('mods'):
        for file in os.listdir('mods'):
            if not os.path.isfile(os.path.join('mods', file)):
                continue
            yield os.path.splitext(os.path.basename(file))[0]
    for file in os.listdir(usermod_dir):
        if not os.path.isfile(os.path.join(usermod_dir, file)):
            continue
        yield os.path.splitext(os.path.basename(file))[0]


def getname(mod):
    if hasattr(mod, 'name'):
        return mod.name
    else:
        return mod.__name__


def modslist(quitter):
    print('The following are the installed mods:')
    for mod in mods:
        mod = getname(mod)
        print('+', mod)
    print('Total mods: %i' % len(mods))


unomain.options.insert(-1, ('Mods', modslist))
usermod_dir = os.path.expanduser('~/.uno_forge_mods')
if not os.path.exists(usermod_dir):
    os.mkdir(usermod_dir)
sys.path.append('mods')
sys.path.append(usermod_dir)
mods = []

unoforge.mods = mods
for mod in getmods():
    mods.append(__import__(mod))

unomain.exit_callback = _exit_callback
if unoforge._cards_to_add:
    uno.CARD_LIST[:], uno.WEIGHT_LIST[:] = uno.calculate_chance()

for mod in mods:
    if hasattr(mod, 'init'):
        mod.init()

if __name__ == '__main__':
    unomain.main()



import setuptools

setuptools.setup(
    name = 'Uno Game-Forge',
    version = '1.1.0',
    url = 'https://github.com/gaming32/uno',
    author = 'Gaming32',
    author_email = 'gaming32i64@gmail.com',
    license = 'License :: OSI Approved :: MIT License',
    description = 'Modding for Uno Game',
    long_description = open('README.md').read(),
    long_description_content_type = 'text/markdown',
    install_requires = [
        'Uno-Game>=1.1b5',
    ],
    py_modules = [
        'uno_forge',
        'unoforge',
    ],
    scripts = [
        'uno_forge.py',
    ],
)


import uno as _uno
import uno.main as _unomain

_exit_callbacks = []


def register_exit_callback(func):
    _exit_callbacks.append(func)


mods = None


def add_menu_option(label, func, index=None):
    """Arguments
---------
label : str
    The label for the option
func : callable
    Callable to call when option is selected; return `True` to exit the menu
index : int, None
    Where to put the option (uses list.insert); if is `None` adds to end (uses list.append)"""
    def menu_option(quitter, func=func):
        ret = func()
        if ret is not None:
            quitter[0] = bool(ret)
    opt = (label, menu_option)
    if index is None:
        _unomain.options.append(opt)
    else:
        _unomain.options.insert(index, opt)


_cards_to_add = set()


def add_single_card(card_class):
    card_obj = card_class()
    _uno.CARD_SET.add(card_obj)
    _cards_to_add.add(card_obj)
    return card_obj


def menu(name, **opts):
    """Note: OrderedDict recommended for `opts` argument"""
    _unomain.menu(name, list(opts.items()))


def add_custom_player_type(label, player_class):
    """`label` is displayed as "How many %s would you like?" Where %s is `label`"""
    _unomain.player_types.append((label, player_class))
